<?php

namespace Drupal\cron_ghost\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\NodeType;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Configure which content types get the Cron Ghost Time field.
 */
class CronGhostSettingsForm extends ConfigFormBase {

  public function getFormId() {
    return 'cron_ghost_settings';
  }

  protected function getEditableConfigNames() {
    return ['cron_ghost.settings'];
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cron_ghost.settings');
    $content_types = NodeType::loadMultiple();

    $options = [];
    foreach ($content_types as $type) {
      $options[$type->id()] = $type->label();
    }

    $form['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select content types to use Cron Ghost with.'),
      '#default_value' => $config->get('content_types') ?? [],
      '#options' => $options,
    ];

    return parent::buildForm($form, $form_state);
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $new_selected = array_filter($form_state->getValue('content_types'));
    $config = $this->config('cron_ghost.settings');
    $old_selected = $config->get('content_types') ?? [];

    $config->set('content_types', $new_selected)->save();

    $added = array_diff($new_selected, $old_selected);
    $removed = array_diff($old_selected, $new_selected);

    foreach ($added as $bundle) {
      if (!FieldStorageConfig::loadByName('node', 'field_cron_ghost_time')) {
        FieldStorageConfig::create([
          'field_name' => 'field_cron_ghost_time',
          'entity_type' => 'node',
          'type' => 'datetime',
          'settings' => ['datetime_type' => 'datetime'],
        ])->save();
      }

      if (!FieldConfig::loadByName('node', $bundle, 'field_cron_ghost_time')) {
        FieldConfig::create([
          'field_name' => 'field_cron_ghost_time',
          'entity_type' => 'node',
          'bundle' => $bundle,
          'label' => 'Unpublish date and time',
          'description' => 'The date and time when this content should be unpublished.',
          'required' => FALSE,
        ])->save();
      }
    }

    foreach ($removed as $bundle) {
      $field = FieldConfig::loadByName('node', $bundle, 'field_cron_ghost_time');
      if ($field) {
        $field->delete();
      }
    }

    parent::submitForm($form, $form_state);
  }
}
